#include "crypto.h"
#include <stdlib.h>  // for random()

Crypto::Crypto(ushort key, QString opseq, ushort charsetsiz) 
: m_Key(key), m_OpSequence(opseq), m_CharSetSize(charsetsiz) { }

QString Crypto::encrypt(const QString& str) {
    randomPerm(str.length());
    QString oldstr(str), newstr;
    for (ushort i = 0; i < m_OpSequence.length(); ++i) {
        if (m_OpSequence[i] == 's')
            newstr = shift(oldstr);
        else
            newstr = permute(oldstr);
        oldstr = newstr;
    }
    return newstr;
}

QString Crypto::decrypt(const QString& str) {
    randomPerm(str.length());
    QString oldstr(str), newstr;
    for (ushort i = m_OpSequence.length(); i > 0;  --i) {
        if (m_OpSequence[i -1] == 's')
            newstr = unshift(oldstr);
        else
            newstr = unpermute(oldstr);
        oldstr = newstr;
    }
    return newstr;  
}


/* shift() works with the QChars of the given QString */
QString Crypto::shift(const QString& text)  const{
    srandom(m_Key);
    const ushort P(m_CharSetSize);
    QString cryptstr;
    ushort ccode, shiftcode;
    int rnd;
    for (int i = 0; i < text.length(); ++i) {
        rnd = limitedRand(m_CharSetSize);
        ccode = text[i].unicode();
        shiftcode = (ccode + rnd) % P ;
        cryptstr[i] = QChar(shiftcode) ;
    }
    return cryptstr;
}

QString Crypto::unshift(const QString& text)  const{
   srandom(m_Key);
   const ushort P(m_CharSetSize);
   QString cryptstr;
   ushort ccode, shiftcode;
   int rnd;
   for (int i = 0; i < text.length(); ++i) {
      rnd = limitedRand(m_CharSetSize);
      ccode = text[i].unicode();
      shiftcode = ((ccode + P) - rnd) % P ;  // make sure it's positive
      cryptstr[i] = QChar(shiftcode) ;
   }
   return cryptstr;
}

int Crypto::limitedRand(int max) {
   return random() % max ;
}

void sequenceFill(QVector<int>& vec) {
   for (int i = 0; i < vec.size(); ++i) 
       vec[i] = i;
}

void swap(int& x, int& y) {
    int temp;
    temp = x;
    x = y;
    y = temp;
}

void Crypto::randomPerm(int n)  {
    const int numswaps(2 * n);
     m_Perm.clear(); 
     m_Perm.resize(n);
     sequenceFill(m_Perm);
     srandom(m_Key);
     int j, k;
     for (int i = 0; i < numswaps; ++i) {
             j = limitedRand(n);
             k = limitedRand(n);
             swap(m_Perm[j], m_Perm[k]);
     }
}

QString Crypto::permute(const QString& text) {
    int slen = text.length();
    QString scrstr;
    for (int i = 0; i < slen; ++i) 
        scrstr += text.at(m_Perm[i]);
    return scrstr;
}

QString Crypto::unpermute(const QString& text) {
    int slen = text.length();
    QString uscrstr;
    for (int i = 0; i < slen; ++i) 
        uscrstr[m_Perm[i]] = text[i];
    return uscrstr;
 }


